@@ -360,17 +360,20 @@ return a list of `llm-chat-prompt-tool-use' structs.")
      :headers (llm-provider-headers provider)
      :data (llm-provider-chat-request provider prompt nil)
      :on-success (lambda (data)
-                   (if-let ((err-msg (llm-provider-chat-extract-error provider data)))
-                       (llm-provider-utils-callback-in-buffer
-                        buf error-callback 'error
-                        err-msg)
-                     (llm-provider-utils-process-result
-                      provider prompt
-                      (llm-provider-utils-extract-all provider data)
-                      multi-output
-                      (lambda (result)
-                        (llm-provider-utils-callback-in-buffer
-                         buf success-callback result)))))
+                   (with-current-buffer (if (buffer-live-p buf)
+                                            buf
+                                          (generate-new-buffer " *llm-temp*" t))
+                     (if-let ((err-msg (llm-provider-chat-extract-error provider data)))
+                         (llm-provider-utils-callback-in-buffer
+                          buf error-callback 'error
+                          err-msg)
+                       (llm-provider-utils-process-result
+                        provider prompt
+                        (llm-provider-utils-extract-all provider data)
+                        multi-output
+                        (lambda (result)
+                          (llm-provider-utils-callback-in-buffer
+                           buf success-callback result))))))
      :on-error (lambda (_ data)
                  (llm-provider-utils-callback-in-buffer
                   buf error-callback 'error
@@ -394,8 +397,11 @@ Any strings will be concatenated, integers will be added, etc."
       (if new
           (progn
             (unless (eq (type-of current) (type-of new))
-              (error "Cannot accumulate different types of streaming results: %s and %s"
-                     current new))
+              (signal
+               'llm-invalid-argument
+               (list (format
+                      "Cannot accumulate different types of streaming results: %s and %s"
+                      current new))))
             (pcase (type-of current)
               ('string (concat current new))
               ('integer (+ current new))
@@ -443,18 +449,20 @@ Any strings will be concatenated, integers will be added, etc."
      :on-success
      (lambda (_)
        ;; We don't need the data at the end of streaming, so we can ignore it.
-       (llm-provider-utils-process-result
-        provider prompt
-        (llm-provider-utils-streaming-accumulate
-         current-result
-         (when-let ((tool-uses-raw (plist-get current-result
-                                              :tool-uses-raw)))
-           `(:tool-uses ,(llm-provider-collect-streaming-tool-uses
-                          provider tool-uses-raw))))
-        multi-output
-        (lambda (result)
-          (llm-provider-utils-callback-in-buffer
-           buf response-callback result))))
+       (with-current-buffer (if (buffer-live-p buf)
+                                buf
+                              (generate-new-buffer " *llm-temp*" t))
+         (llm-provider-utils-process-result
+          provider prompt
+          (llm-provider-utils-streaming-accumulate
+           current-result
+           (when-let ((tool-uses-raw (plist-get current-result
+                                                :tool-uses-raw)))
+             `(:tool-uses ,(llm-provider-collect-streaming-tool-uses
+                            provider tool-uses-raw))))
+          multi-output
+          (lambda (result)
+            (llm-provider-utils-callback-in-buffer buf response-callback result)))))
      :on-error (lambda (_ data)
                  (llm-provider-utils-callback-in-buffer
                   buf error-callback 'error
@@ -710,7 +718,7 @@ This returns a JSON object (a list that can be converted to JSON)."
 (defun llm-provider-utils-append-to-prompt (prompt output &optional tool-results role)
   "Append OUTPUT to PROMPT as an assistant interaction.
 
-OUTPUT can be a string or a structure in the case of function calls.
+OUTPUT can be a string or a structure in the case of tool uses.
 
 TOOL-RESULTS is a list of results from the LLM output, if any.
 
@@ -747,7 +755,8 @@ SUCCESS-CALLBACK.
 
 SUCCESS-CALLBACK is the callback that will be run when all functions
 complete."
-  (when (plist-get partial-result :text)
+  (when (and (plist-get partial-result :text)
+             (> (length (plist-get partial-result :text)) 0))
     (llm-provider-append-to-prompt provider prompt (plist-get partial-result :text)))
   (if-let ((tool-uses (plist-get partial-result :tool-uses)))
       ;; If we have tool uses, execute them, and on the callback, we will
@@ -814,7 +823,8 @@ This will convert all :json-false and :false values to nil."
    ((member args '(:json-false :false)) nil)
    (t args)))
 
-(defun llm-provider-utils-execute-tool-uses (provider prompt tool-uses multi-output partial-result success-callback)
+(defun llm-provider-utils-execute-tool-uses (provider prompt tool-uses multi-output
+                                                      partial-result success-callback)
   "Execute TOOL-USES, a list of `llm-provider-utils-tool-use'.
 
 A response suitable for returning to the client will be returned.
@@ -839,14 +849,22 @@ have returned results."
      for tool-use in tool-uses do
      (let* ((name (llm-provider-utils-tool-use-name tool-use))
             (arguments (llm-provider-utils-tool-use-args tool-use))
-            (tool (seq-find
-                   (lambda (f) (equal name (llm-tool-name f)))
-                   (llm-chat-prompt-tools prompt)))
+            (tool (or
+                   (seq-find
+                    (lambda (f) (equal name (llm-tool-name f)))
+                    (llm-chat-prompt-tools prompt))
+                   (signal 'llm-tool-unknown-tool `(:tool ,name))))
             (call-args (cl-loop for arg in (llm-tool-args tool)
-                                collect (cdr (seq-find (lambda (a)
-                                                         (eq (intern (plist-get arg :name))
-                                                             (car a)))
-                                                       arguments))))
+                                collect (cdr (or
+                                              (seq-find (lambda (a)
+                                                          (eq (intern (plist-get arg :name))
+                                                              (car a)))
+                                                        arguments)
+                                              ;; Arg wasn't found, if it wasn't
+                                              ;; optional, signal an error.
+                                              (unless (plist-get arg :optional)
+                                                (signal 'llm-tool-missing-argument
+                                                        `((:tool ,name :arg ,arg))))))))
             (end-func (lambda (result)
                         (llm--log
                          'api-funcall
@@ -865,6 +883,15 @@ have returned results."
                                         (append partial-result
                                                 `(:tool-results ,tool-use-and-results)))
                                      tool-use-and-results))))))
+       ;; Check to see that there were no unknown args.
+       (dolist (arg-key (map-keys arguments))
+         (unless (seq-find
+                  (lambda (a) (eq (intern (plist-get a :name))
+                                  arg-key))
+                  (llm-tool-args tool))
+           (signal 'llm-tool-unknown-argument
+                   `((:tool ,name :arg ,arg-key)))))
+
        (if (llm-tool-async tool)
            (apply (llm-tool-function tool)
                   (append (list end-func) call-args))
