@@ -104,7 +104,11 @@ information than standard tool use."
                   (float-time (time-subtract (current-time) (or (llm-vertex-key-gentime provider) 0)))))
     (let ((result (string-trim (shell-command-to-string (concat llm-vertex-gcloud-binary " auth print-access-token")))))
       (when (string-match-p "ERROR" result)
-        (error "Could not refresh gcloud access token, received the following error: %s" result))
+        (signal
+         'llm-provider-error
+         (list (format
+                "Could not refresh gcloud access token, received the following error: %s"
+                result))))
       ;; We need to make this unibyte, or else it doesn't causes problems when
       ;; the user is using multibyte strings.
       (setf (llm-vertex-key provider) (encode-coding-string result 'utf-8)))
@@ -250,15 +254,30 @@ information than standard tool use."
    (when (llm-chat-prompt-tools prompt)
      ;; Although Gemini claims to be compatible with Open AI's function declaration,
      ;; it's only somewhat compatible.
-     `(:tools
-       [(:function_declarations
-         ,(vconcat (mapcar
-                    (lambda (tool)
-                      `(:name ,(llm-tool-name tool)
-                              :description ,(llm-tool-description tool)
-                              :parameters ,(llm-provider-utils-openai-arguments
-                                            (llm-tool-args tool))))
-                    (llm-chat-prompt-tools prompt))))]))
+     (append
+      `(:tools
+        [(:function_declarations
+          ,(vconcat (mapcar
+                     (lambda (tool)
+                       `(:name ,(llm-tool-name tool)
+                               :description ,(llm-tool-description tool)
+                               :parameters ,(llm-provider-utils-openai-arguments
+                                             (llm-tool-args tool))))
+                     (llm-chat-prompt-tools prompt))))])
+      (when-let* ((options (llm-chat-prompt-tool-options prompt)))
+        `(:toolConfig
+          (:functionCallingConfig
+           ,(append
+             (list :mode (pcase (llm-tool-options-tool-choice options)
+                           ('auto "AUTO")
+                           ('none "NONE")
+                           ('any "ANY")
+                           ((pred stringp) "ANY")
+                           (_ (signal
+                               'llm-not-supported
+                               (list (format "Unknown tool choice option: %s" (llm-tool-options-tool-choice options)))))))
+             (when (stringp (llm-tool-options-tool-choice options))
+               `(:allowedFunctionNames [,(llm-tool-options-tool-choice options)]))))))))
    (llm-vertex--chat-parameters prompt model)))
 
 ;; TODO: remove after September 2025, this is only here so people can upgrade to
